using System;
using Gtk;
using Foodolini.BusinessLogic;
using Foodolini.Activities;

namespace Foodolini.Activities.Cookbook
{
	/// <summary>
	/// Adapts a person to the TreeNode interface
	/// </summary>
	[TreeNode (ListOnly = true)]
	public class PersonAdapter : TreeNode
	{
		/// <summary>
		/// Sets the person and the percentage eaten
		/// </summary>
		/// <param name="person">
		/// A <see cref="Person"/>
		/// </param>
		/// <param name="percentageEaten">
		/// A <see cref="System.Double"/>
		/// </param>
		public PersonAdapter(Person person, double percentageEaten)
		{
			this.Person = person;
			this.PercentageEaten = percentageEaten;
		}
		
		/// <summary>
		/// Set the person
		/// </summary>
		/// <param name="person">
		/// A <see cref="Person"/>
		/// </param>
		public PersonAdapter(Person person)
		{
			this.Person = person;
		}
		
		/// <value>
		/// Getter for the full name of the person
		/// </value>
		[Gtk.TreeNodeValue (Column=0)]
		public string Name{
			get{
				return this.Person.FullName;
			}
		}
		
		/// <value>
		/// Getter for the user name of the person
		/// </value>
		[Gtk.TreeNodeValue (Column=2)]
		public string UserName{
			get{
				return this.Person.UserName;
			}
		}
		
		/// <value>
		/// Getter and setter of the percentage eaten of the meal
		/// </value>
		[Gtk.TreeNodeValue (Column=1)]
		public double PercentageEaten{ get; set;}
		
		/// <value>
		/// Getter and private setter of the person
		/// </value>
		public Person Person{get; private set;}
	}
}
