
using System;
using Gtk;
using Foodolini.Activities;
using Foodolini.BusinessLogic;

namespace Foodolini.Activities.Cookbook
{
	
	[TreeNode (ListOnly = true)]
	public class RecipeAdapter : TreeNode
	{
		/// <summary>
		/// Constructor that sets the recipe
		/// </summary>
		/// <param name="recipe">
		/// A <see cref="Recipe"/>
		/// </param>
		public RecipeAdapter(Recipe recipe)
		{
			this.Recipe = recipe;
			this.Type = "Unknown";
			foreach (var category in recipe.Categories)
			{
				if(category == "Breakfast" | category == "Dinner" | category == "Lunch" | category == "Other")
					this.Type = category;
			}
			this.Rating = recipe.AverageRating;
		}
		
		/// <value>
		/// Getter for the title of the recipe
		/// </value>
		[Gtk.TreeNodeValueAttribute(Column=0)]
		public string Title{
			get{return Recipe.Title;}
		}
		
		/// <value>
		/// Getter and private setter for the meal type
		/// </value>
		[Gtk.TreeNodeValueAttribute(Column=2)]
		public string Type{get; private set;}
		
		/// <value>
		/// Getter and private setter for the averageRating.
		/// </value>
		[Gtk.TreeNodeValueAttribute(Column=1)]
		public double Rating{get; private set;}
		
		
		/// <value>
		/// Getter and private setter of the recipe
		/// </value>
		public Recipe Recipe{get; private set;}
	}
}
