
using System;
using Foodolini.Activities;
using Gtk;
using Foodolini.BusinessLogic;
using System.Collections.Generic;

namespace Foodolini.Activities.Cookbook
{
	
	/// <summary>
	/// An activity that show a selected recipe
	/// </summary>
	[FoodoliniActivity("View recipe", true)]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class RecipeViewer : Gtk.Bin, IActivity
	{
		private IOwner owner;
		private Recipe recipe;
		private Gdk.Pixbuf pxStarHide = Gdk.Pixbuf.LoadFromResource("StarHide.png");
		private Gdk.Pixbuf pxStar = Gdk.Pixbuf.LoadFromResource("Star.png");
		private NodeStore ingredientStorage = new NodeStore(typeof(IngredientAdapter));
		private Dictionary<Ingredient, double> ingredientSave = new Dictionary<Ingredient, double>();
		private List<FoodItem> fooditemsStorage;
		
		private Image star1;
		private Image star2;
		private Image star3;
		private Image star4;
		private Image star5;
		private Image imgRecipeBigVersion;
		
		
		/// <summary>
		/// Constructer that sets the images for rating and sets up the nodeview
		/// </summary>
		public RecipeViewer()
		{
			this.Build();
			imgRecipeBigVersion = new Image();
			
			fooditemsStorage = new List<FoodItem>(FoodItem.ListFoodItems());
			
			this.Destroyed += HandleDestroyed;
			
			star1 = new Image(pxStarHide);
			star2 = new Image(pxStarHide);
			star3 = new Image(pxStarHide);
			star4 = new Image(pxStarHide);
			star5 = new Image(pxStarHide);
			btnStar1.Image = star1;
			btnStar2.Image = star2;
			btnStar3.Image = star3;
			btnStar4.Image = star4;
			btnStar5.Image = star5;
			bxViewRecipe.ShowAll();
			
			//Set consume and store buttons to false as default. If all ingredients are present as fooditems, they will be set to true
			btnConsume.Sensitive = false; btnStore.Sensitive = false;
			btnAdd2ShoppingList.Sensitive = false;
			
			
			//Set up nodeview
			this.nvIngredients.NodeStore = this.ingredientStorage;
			this.nvIngredients.AppendColumn("Ingredients", new CellRendererText(), (tree_column, cell, node)=>{
				IngredientAdapter n = (IngredientAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Name;
			});
			this.nvIngredients.AppendColumn("Quantity (g)", new CellRendererText(), (tree_colum, cell, node)=>{
				IngredientAdapter n = (IngredientAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Quantity.ToString();
			});			
			this.nvIngredients.AppendColumn("Storage", new CellRendererText(), (tree_colum, cell, node)=>{
				IngredientAdapter n = (IngredientAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Storage;
			});
			
			this.nvIngredients.AppendColumn("", new CellRendererText(), (tree_colum, cell, node)=>{
				IngredientAdapter n = (IngredientAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Expiration;
			});
		}
		
		public event EventHandler<RecipeEventArgs> OnRecipeViewed;
		public event EventHandler<RecipeEventArgs> OnRecipeEdited;
		public event EventHandler<RecipeEventArgs> OnRecipeDeleted;
	
		private void CheckIngredients()
		{
			BusinessLogic.Settings.Instance.Save();
			double temp = 0;
			nvIngredients.NodeStore = null;
			foreach(IngredientAdapter node in ingredientStorage)
			{
				node.Quantity = ingredientSave[node.Ingredient] * sbMultiplier.Value;
				temp = -node.Quantity;
			
				FoodItem fooditem = node.Ingredient.InStorage();	
				if(fooditem != null)
				{
					temp = fooditem.Quantity - node.Quantity;
					if((fooditem.ExpirationDate - DateTime.Now).Days < 0)
						node.Expiration = "Expired";
				}
				
				if (temp < 0)
				{
					node.Storage = (-temp).ToString() + " g missing";
					btnConsume.Sensitive = false; btnStore.Sensitive = false;
				}
				else if (temp == 0)
				{
					node.Storage = temp.ToString();
					btnConsume.Sensitive = true; btnStore.Sensitive = true;
				}
				else if(temp > 0)
				{
					node.Storage = temp.ToString() + " g remaining";
					btnConsume.Sensitive = true; btnStore.Sensitive = true;
				}
			}
			this.nvIngredients.NodeStore = this.ingredientStorage;
			bxViewRecipe.ShowAll();
		}
		
		/// <summary>
		/// Destroy event that disposes all the images.
		/// </summary>
		void HandleDestroyed(object sender, EventArgs e)
		{
			this.star1.Dispose();
			this.star2.Dispose();
			this.star3.Dispose();
			this.star4.Dispose();
			this.star5.Dispose();	
			if(this.recipe.Picture != null)
				this.imgRecipe.Image.Dispose();
			if(imgRecipeBigVersion.Pixbuf != null)
				this.imgRecipeBigVersion.Dispose();			
		}
		
		/// <summary>
		/// Deletes the current recipe
		/// </summary>
		protected virtual void OnDeleteRecipeClick (object sender, System.EventArgs e)
		{
			ConfirmDialog dialog = new ConfirmDialog("Delete recipe", "Are you sure you want to delete " + this.recipe.Title);
			
			if (dialog.Run() == (int)ResponseType.Ok)
			{
				this.recipe.Delete();
				this.Destroy();
				if(OnRecipeDeleted != null)
					OnRecipeDeleted(this, new RecipeEventArgs(this.recipe));
			}
			dialog.Destroy();
		}
		
		/// <summary>
		/// Changes the amounts on the ingredients according to the number of servings chosen.
		/// </summary>
		protected virtual void OnSbMultiplierChanged (object sender, System.EventArgs e)
		{
			CheckIngredients();
		}
		
		/// <summary>
		/// Saves the selected rating on the Recipe. Buttons are changed according to rating.
		/// Rate on Recipe not implemented.
		/// </summary>
		protected virtual void RateRecipe(object sender, System.EventArgs e){
			int rating = 0;
			
			switch(((Button)sender).Name){
			case "btnStar1":
				rating = 1;
				break;
			case "btnStar2":
				rating = 2;
				break;
			case "btnStar3":
				rating = 3;
				break;
			case "btnStar4":
				rating = 4;
				break;
			case "btnStar5":
				rating = 5;
				break;
			default:
				break;
			}

			if(this.owner.CurrentUser != null)
			{
				this.recipe.Rate(this.owner.CurrentUser, rating);
				SetRating();
				lbAverageRating.Text = this.recipe.AverageRating.ToString("0.00");
				this.recipe.Save();
			}
			if(OnRecipeViewed != null)
				OnRecipeViewed(this, new RecipeEventArgs(this.recipe));
			
		}
		
		/// <summary>
		/// Method used to set the recipe to be viewed. Also views the detials of the recipe.
		/// </summary>
		/// <param name="recipe">
		/// A <see cref="Recipe"/>
		/// </param>
		public void SetParamenters(Recipe recipe){
			this.recipe = recipe;
			SetRecipeDetails();
			SetRating();
			CheckUser();

		}
		
		/// <summary>
		/// Set the pixbuffer on the images for rating
		/// </summary>
		private void SetRatingButtons(Gdk.Pixbuf px1, Gdk.Pixbuf px2, Gdk.Pixbuf px3, Gdk.Pixbuf px4, Gdk.Pixbuf px5){
			
			star1.Pixbuf = px1;
			star2.Pixbuf = px2;
			star3.Pixbuf = px3;
			star4.Pixbuf = px4;
			star5.Pixbuf = px5;
		}
		
		private void SetRating()
		{
			CheckUser();
			if (this.owner.CurrentUser != null)
			{
				if (this.recipe.Ratings.ContainsKey(this.owner.CurrentUser))
				{
					double rating  = this.recipe.Ratings[this.owner.CurrentUser];
					if ( rating == 1)
						SetRatingButtons(pxStar,pxStarHide,pxStarHide,pxStarHide,pxStarHide);
					else if (rating == 2)
						SetRatingButtons(pxStar,pxStar,pxStarHide,pxStarHide,pxStarHide);
					else if (rating == 3)
						SetRatingButtons(pxStar,pxStar,pxStar,pxStarHide,pxStarHide);
					else if(rating == 4)
						SetRatingButtons(pxStar,pxStar,pxStar,pxStar,pxStarHide);
					else if(rating == 5)
						SetRatingButtons(pxStar,pxStar,pxStar,pxStar,pxStar);
				}
				else
					SetRatingButtons(pxStarHide,pxStarHide,pxStarHide,pxStarHide,pxStarHide);
			}
			else
				SetRatingButtons(pxStarHide,pxStarHide,pxStarHide,pxStarHide,pxStarHide);
		}
	
		void OwnerhandleCurrentUserChanged (object sender, PersonEventArgs e)
		{
			SetRating();
		}
		
		private void CheckUser()
		{
			if(this.owner.CurrentUser == null)
			{
				btnAdd2ShoppingList.Sensitive = false;
				btnStar1.Sensitive = false; btnStar2.Sensitive = false;
				btnStar3.Sensitive = false; btnStar4.Sensitive = false; 
				btnStar5.Sensitive = false; 
			}
			else
			{
				btnAdd2ShoppingList.Sensitive = true;
				btnStar1.Sensitive = true; btnStar2.Sensitive = true;
				btnStar3.Sensitive = true; btnStar4.Sensitive = true; 
				btnStar5.Sensitive = true; 			
			}
		}
			
		private Button imgRecipe = new Button();
		/// <summary>
		/// Views the details of the recipe
		/// </summary>
		private void SetRecipeDetails(){
			//View title
			lbTitle.Markup = "<big><b>" + recipe.Title + "</b></big>";
			imgRecipe.TooltipText="Click for a larger picture of "+recipe.Title;
			//View ingredients
			ingredientStorage.Clear();
			ingredientSave.Clear();
			
			foreach(KeyValuePair<Ingredient, double> ingredient in this.recipe.Ingredients)
			{
				IngredientAdapter ia = new IngredientAdapter(ingredient.Key, ingredient.Value);
				this.ingredientStorage.AddNode(ia);
				this.ingredientSave.Add(ingredient.Key, ingredient.Value);
			}
			
			//Set number of servings
			sbMultiplier.Value = this.recipe.Servings;
			
			CheckIngredients();
			
			//View directions
			string recipeView = "";
			foreach(var direction in this.recipe.Directions)
				recipeView += direction + "\n";
			tvViewRecipe.Buffer.Text = recipeView;
			
			//View picture - if any - in a button that opens a bigger version.
			if(recipe.Picture != null)
			{
				Gdk.Pixbuf pxr = new Gdk.Pixbuf(recipe.Picture);
				pxr = pxr.ScaleSimple(100, 100, Gdk.InterpType.Bilinear);//TODO: Make prettier
				imgRecipe.Image = new Image(pxr);
				imgRecipe.Relief = ReliefStyle.None;
				imgRecipe.TooltipText = "Opens a bigger version.";
				imgRecipe.Clicked += ShowBigVersion;
				bxBottom.Add(imgRecipe);
			}	
			
			//Set average rating, difficulty and preparation time
			lbAverageRating.Text = this.recipe.AverageRating.ToString("0.00");
			lbDifficulty.Text = this.recipe.Difficulty.ToString();
			TimeSpan preparationTime = this.recipe.PreparationTime;//TOODO: Remove s when 1
			
			string prepFormat = string.Empty;
			
			if(preparationTime.Days != 0) {
				prepFormat += preparationTime.Days + " day";
				
				if(preparationTime.Days > 1)
					prepFormat += "s";
				
				if(preparationTime.Hours != 0 || preparationTime.Minutes != 0)
					prepFormat += ", ";
				
			}
			if(preparationTime.Hours != 0) {
				prepFormat += preparationTime.Hours + " hour";
				
				if(preparationTime.Hours > 1)
					prepFormat += "s";
				
				if(preparationTime.Minutes != 0)
					prepFormat += ", ";
			}
			if(preparationTime.Minutes != 0) {
				prepFormat +=preparationTime.Minutes + " minute";
				
				if(preparationTime.Minutes > 1)
					prepFormat += "s";
				
			}
			lbTime.Text =   prepFormat;
		}

		Window w = new Window(WindowType.Toplevel);
		/// <summary>
		/// Opens a big version of the image in a new window
		/// </summary>
		protected virtual void ShowBigVersion (object sender, System.EventArgs e)
		{
			w.Title = this.recipe.Title;
			Gdk.Pixbuf pxr = new Gdk.Pixbuf(recipe.Picture);
			pxr = pxr.ScaleSimple(500, 500, Gdk.InterpType.Bilinear);//TODO: Make prettier
			imgRecipeBigVersion.Pixbuf = pxr;
			w.Add(imgRecipeBigVersion);
			w.ShowAll();
		}
		
		#region New activities
		/// <summary>
		/// Opens a dialog where the people who attended the meal can be selected,
		/// and saves this on the selected people. Any leftovers are stored.
		/// </summary>
		protected virtual void OnConsumeClick (object sender, System.EventArgs e)
		{
			Dictionary<Person, double> users = new Dictionary<Person, double>();
			double leftover = 100;
			FoodItem foodItem = null;
			UserSelector dialog = new UserSelector();
			if (dialog.Run() == (int)ResponseType.Ok)
			{
				users = dialog.GetUsers();
							
				foreach(KeyValuePair<Person, double> user in users)
				{
					leftover -= user.Value;
					foodItem = this.recipe.Cook(sbMultiplier.Value, user.Value );
					foodItem.Consume(user.Key);
					foodItem.Save();
				}	
				if(leftover > 0)
				{
					foodItem = this.recipe.Cook(sbMultiplier.Value, leftover);
					foodItem.Save();
				}
			}
			dialog.Destroy();
			
			//Updates the fooditems in storage
			fooditemsStorage = new List<FoodItem>(FoodItem.ListFoodItems());
		}
		
		
		/// <summary>
		/// Opens a RecipeEditor where the current recipe can be edited
		/// </summary>
		protected virtual void EditRecipe (object sender, System.EventArgs e)
		{
			RecipeEditor re = this.owner.PushActivity<RecipeEditor>();
			re.SetParamenters(this.recipe);
			re.OnRecipeEdited += HandleOnRecipeEdited;
			
		}

		/// <summary>
		/// Updates the RecipeViewer after being editted.
		/// </summary>
		void HandleOnRecipeEdited(object sender, RecipeEventArgs e)
		{
			this.recipe = e.Recipe;
			SetRecipeDetails();
			if(this.OnRecipeEdited != null)
			{
				this.OnRecipeEdited(this, new RecipeEventArgs(this.recipe));
			}
			//Foodolini.BusinessLogic.Settings.Instance.Save();
		}
		
		/// <summary>
		/// Converts the recipe to a food item, and stores it
		/// Not Implemented
		/// </summary>
		protected virtual void OnStoreClick (object sender, System.EventArgs e)
		{
			ConfirmDialog dialog = new ConfirmDialog("Store recipe", "Do you want to store food made\nfrom this recipe in your inventory?");
						
			if (dialog.Run() == (int)ResponseType.Ok)
			{
				this.recipe.Cook(sbMultiplier.Value).Save();
			}
			dialog.Destroy();
			fooditemsStorage = new List<FoodItem>(FoodItem.ListFoodItems());
			SetRecipeDetails();
		}
		
		#endregion
		
		#region IActivity
		public void Register(IOwner owner){
			this.owner = owner;
			this.owner.CurrentUserChanged += OwnerhandleCurrentUserChanged;
		}
		
		public void Unregister(){
			this.owner.CurrentUserChanged -= OwnerhandleCurrentUserChanged;
			this.owner = null;
		}
		
		public Widget Widget{
			get{
				return this;
			}
		}
		#endregion

		protected virtual void OnBtnRecipe2ShoppingListClicked (object sender, System.EventArgs e)
		{
			this.owner.AddRecipe(this.recipe, sbMultiplier.Value);
			this.owner.ToggleShoppingList(true);
		}

		protected virtual void OnBtnCloseViewClicked (object sender, System.EventArgs e)
		{
			this.Destroy();
		}
		
	}
}
