//Compile this file for Windows support
using System;
using System.Collections.Generic;
using System.Text;
using System.Runtime.InteropServices;
using DirectShowLib;

namespace Foodolini.Activities.FoodRegistration.Devices
{

	public class WindowsCaptureDeviceMonitor : ICaptureDeviceMonitor
	{
		private WindowsCaptureDeviceMonitor ()
		{

		}

		public bool SupportsAutoRefresh {
			get { return false; }
		}

		#region Singleton implementation

		/// <summary>
		/// The singleton instance holder
		/// </summary>
		private static WindowsCaptureDeviceMonitor instance = null;

		public static WindowsCaptureDeviceMonitor Instance {
			get {
				if (instance == null)
					instance = new WindowsCaptureDeviceMonitor ();
				return instance;
			}
		}

		#endregion

		#region ICaptureDeviceMonitor implementation

		public event EventHandler<DeviceEventArgs> DeviceAdded;

		public event EventHandler<DeviceEventArgs> DeviceRemoved;

		public System.Collections.Generic.ICollection<IDeviceInfo> Devices {
			get {
				var retval = new List<IDeviceInfo> ();
				// List devices
				int numDevices=0;
				foreach (DsDevice device in DsDevice.GetDevicesOfCat (FilterCategory.VideoInputDevice)) {
					//retval.Add(new DeviceEventArgs(device.Name, device.DevicePath));
					numDevices++;
				}
				
				if(numDevices > 0) {
					StringBuilder name = new StringBuilder (256);
					StringBuilder desc = new StringBuilder (256);
					for (int i = 0; i < 9; i++) {
						if (capGetDriverDescription (i, name, (uint)name.Capacity, desc, (uint)desc.Capacity)) {
							retval.Add (new DeviceEventArgs (name.ToString () + " (" + desc.ToString () + ")", name.ToString ()));
						}
					}
				}
				
				return retval;
			}
		}

		/// <summary>
		/// Interop call to the old windows API 
		/// </summary>
		/// <param name="driverIndex">
		/// A <see cref="System.Int32"/>
		/// </param>
		/// <param name="name">
		/// A <see cref="StringBuilder"/>
		/// </param>
		/// <param name="nameLength">
		/// A <see cref="System.UInt32"/>
		/// </param>
		/// <param name="description">
		/// A <see cref="StringBuilder"/>
		/// </param>
		/// <param name="descLength">
		/// A <see cref="System.UInt32"/>
		/// </param>
		/// <returns>
		/// A <see cref="System.Boolean"/>
		/// </returns>
		[DllImport("Avicap32.dll")]
		private static extern bool capGetDriverDescription (int driverIndex, StringBuilder name, uint nameLength, StringBuilder description, uint descLength);

		#endregion

		#region IDisposable implementation
		public void Dispose ()
		{
			//Do nothing
		}
		#endregion
	}

	/// <summary>
	/// Static class for getting an instance of ICaptureDeviceMonitor for the current platform
	/// </summary>
	public static class CaptureDeviceMonitor
	{
		public static ICaptureDeviceMonitor Instance {
			get { return WindowsCaptureDeviceMonitor.Instance; }
		}
	}
}
