
using System;
using Gtk;
using Foodolini.BusinessLogic;
using Foodolini.Activities;
using Foodolini.Activities.Ingredients;

namespace Foodolini.Activities.Inventory
{
	/// <summary>
	/// Dialog for editing a FoodItem.
	/// </summary>
	[FoodoliniActivity("EditFoodItem",true)]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class EditFoodItemDialog : Gtk.Dialog
	{

		FoodItem foodItem;
		Ingredient activeIngredient;
		DateTime SavedEx;
		bool Savedisopen;
		
		/// <summary>
		/// Setup the dialog to reflect the given FoodItem.
		/// </summary>
		/// <param name="food">
		/// A <see cref="FoodItem"/> to edit.
		/// </param>
		public EditFoodItemDialog(FoodItem food)
		{
			this.Build();
			
			this.foodItem = food;
			
			activeIngredient = food.Ingredient;
			lbIngredient.Text = foodItem.Ingredient.LongDescription; 
			//bntIngre.Label = foodItem.Ingredient.LongDescription; 
			spinAmount.Value = foodItem.Quantity;
			SavedEx = foodItem.ExpirationDate; 
			cal.Date = foodItem.ExpirationDate;
			Savedisopen = foodItem.IsOpen; 
			chOpen.Active = foodItem.IsOpen;  
			this.Title=foodItem.Ingredient.LongDescription;
		}
		
		
		/// <value>
		/// returns the FoodItem based on the dialog controllers
		/// </value>
		public FoodItem Food{
			get{
				
				this.foodItem.Ingredient = activeIngredient;
				this.foodItem.Quantity = this.spinAmount.Value;
				this.foodItem.ExpirationDate = this.cal.Date;
				
				return this.foodItem; 
			}
		}
		
		
		/// <summary>
		/// return the fooditem to previus settings
		/// </summary>
		public void DontSave(){
			if(Savedisopen){
				foodItem.OpenFoodItem();
			}else{
				foodItem.CloseFoodItem();	
			}
			foodItem.ExpirationDate = SavedEx;
		}
			/// <summary>
			/// Opens a dialog to select an Ingredient.
			/// </summary>
			/// <param name="sender">
			/// A <see cref="System.Object"/>
			/// </param>
			/// <param name="e">
			/// A <see cref="System.EventArgs"/>
			/// </param>
		protected virtual void OnBntIngreClicked (object sender, System.EventArgs e)
		{
			IngredientSelector dialog = new IngredientSelector(activeIngredient,false);
			if (dialog.Run() == (int)ResponseType.Ok){
				activeIngredient = dialog.SelectedIngredient; 
				//this.bntIngre.Label = this.Title = activeIngredient.ShortDescription;
				lbIngredient.Text = this.Title = activeIngredient.LongDescription;
			}
			dialog.Destroy(); 
		}
		
		/// <summary>
		/// Opens or close the FoodItem based on the Toggled
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnChOpenToggled (object sender, System.EventArgs e)
		{
			if(chOpen.Active){
				foodItem.OpenFoodItem();
				cal.Date = foodItem.ExpirationDate;
			}else{
				foodItem.CloseFoodItem();
				cal.Date = foodItem.ExpirationDate;
			}
		}
	}
}
