
using System;
using Foodolini.Activities;
//using Foodolini.Activities.RegisterExercise;
using Foodolini.BusinessLogic;
using Gtk;

namespace Foodolini.Activities.Person
{
	
	[FoodoliniActivity("User profile", false, "profile.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class PersonViewer : Gtk.Bin, IActivity
	{	
		private IOwner owner;
		private Foodolini.BusinessLogic.Person currentUser;		
		private NodeStore exerciseList = new NodeStore(typeof(ExerciseAdapter));
		
			
		public PersonViewer()
		{
			this.Build();
			
			this.ExerciseView.NodeStore = exerciseList;
			this.ExerciseView.AppendColumn("Activity", new CellRendererText(),"text", 0);
			this.ExerciseView.AppendColumn("Duration", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				ExerciseAdapter n = (ExerciseAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text =  "Hours: " + n.Duration.Hours + " and minutes: " + n.Duration.Minutes;
			});
	
			this.ExerciseView.AppendColumn("Date", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				ExerciseAdapter n = (ExerciseAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.PerformedDate.ToShortDateString();
			});
			
			btEditEx.Sensitive = false;
			btDeleteEx.Sensitive = false;
						
			this.Shown += HandleShown;
		}
		
		public void HandleShown(object sender, EventArgs e)
		{
			
			UpDateUser();
		}

		private void UpDateUser(){
			
			if(this.owner.CurrentUser!= null){
				
				currentUser = this.owner.CurrentUser;
				lbFullName.Markup = "<b><big>" + currentUser.FullName + "</big></b>";
				lbUserName.Text = currentUser.UserName;
				lbDateBirth.Text = currentUser.BirthDate.ToShortDateString();
				lbWeight.Text = currentUser.Weight.ToString()+ " kg.";
				lbHeight.Text = currentUser.Height.ToString()+ " cm.";
				lbDiet.Hide();//TODO: As Diet is not yet implemented this is hided
				label7.Hide();//TODO: As Diet is not yet implemented this is hided
				//lbDiet.Text = currentUser.Diet.Name; TODO: As Diet is not yet implemented this is hided
				button314UserProf.Sensitive = true;
				btChangeDiet.Hide();
				vbox5ViewExercises.Show();
				
				this.PopulateExerciseList();
				
						
				}

			else{
				lbFullName.Markup ="<b><big>Anonymous</big></b>";
				lbUserName.Text =  "Anonymous";
				lbDateBirth.Text = "";				
				lbWeight.Text = "";
				lbHeight.Text = "";
				label7.Hide();
				lbDiet.Text = "";
				
				button314UserProf.Sensitive = false;
				btChangeDiet.Visible = false;
				btChangeDiet.Sensitive = false;
				
				vbox5ViewExercises.Hide();
				}
			}
			
		private void HandleUserChanged(object sender, PersonEventArgs e){
		
			UpDateUser();		
		}
		
		/// <summary>
		/// Makes some buttons and the Exercise-browser usable when
		/// the user logs in
		/// </summary>
		public void Register(IOwner owner){
			this.owner = owner;
			this.owner.CurrentUserChanged += HandleUserChanged;
		}
		
		public void Unregister(){
			
			this.owner.CurrentUserChanged -= HandleUserChanged;
			this.owner = null;
		}
		
		// Button actions.
		/// <summary>
		/// Edits user profile
		/// </summary>
			protected virtual void btEditUserProfileOnClick (object sender, System.EventArgs e)
		{
			PersonEditorDialog dia = new PersonEditorDialog();
			dia.SetDetails(this.owner.CurrentUser);
			ResponseType res = (ResponseType)dia.Run();
			
			if(res == ResponseType.Ok){
				this.UpDateUser();
								
			}
			dia.Destroy();
			
			
		}
					
		protected virtual void CancelOnClick (object sender, System.EventArgs e)
		{
			this.owner.PopActivity();
		}

		/// <summary>
		/// Opens up RegisterExerciseDialog for the user to add new exercise
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void RegExOnClicked (object sender, System.EventArgs e)
		{
			RegisterExercise.RegisterExerciseDialog regExercise = new RegisterExercise.RegisterExerciseDialog();
			regExercise.Register(owner);
			if (regExercise.Run() == (int)ResponseType.Ok) {
				this.exerciseList.AddNode(new ExerciseAdapter(regExercise.Exercise));
			}
			regExercise.Unregister();
			regExercise.Destroy();
			
		}
		
		/// <summary>
		/// Edits an exercise chosen in the nodeview
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void EditExOnClicked (object sender, System.EventArgs e)
		{
			if(ExerciseView.NodeSelection.SelectedNode != null) {	
				RegisterExercise.ExerciseEditorDialog exEdit = 
					new RegisterExercise.ExerciseEditorDialog(((ExerciseAdapter)ExerciseView.NodeSelection.SelectedNode).Exercise);
				exEdit.Register(owner);
				if(exEdit.Run() == (int)ResponseType.Ok) {
					exEdit.SaveExercise();
					this.PopulateExerciseList();
				}
				exEdit.Unregister();
				exEdit.Destroy();
			}
			
		}
		/// <summary>
		/// Deletes the exercise marked in the nodeview
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtDeleteExClicked (object sender, System.EventArgs e)
		{
			if(ExerciseView.NodeSelection.SelectedNode != null) {	
				RegisterExercise.ConfirmDialog dialog = new RegisterExercise.ConfirmDialog();
				
				ResponseType res = (ResponseType)dialog.Run();
		
		        if (res == ResponseType.Ok) {
					((ExerciseAdapter)ExerciseView.NodeSelection.SelectedNode).Exercise.Delete();
					this.exerciseList.RemoveNode(ExerciseView.NodeSelection.SelectedNode);
				}
					
				dialog.Destroy();
			}
		}
		
		/// <summary>
		/// Updates the list of exercises performed by the logged-in user
		/// </summary>
		private void PopulateExerciseList() {
			this.exerciseList.Clear();
				foreach(Exercise ex in owner.CurrentUser.GetPerformedExercises())
					this.exerciseList.AddNode(new ExerciseAdapter(ex));
			
		}
		
		/// <summary>
		/// Checks if an exercise is marked in the nodeview.
		/// If it is not, the buttons "Edit" and "Delete" cannot be used.
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnExerciseViewCursorChanged (object sender, System.EventArgs e)
		{
			if (ExerciseView.NodeSelection.SelectedNode != null) {
				btEditEx.Sensitive = true;
				btDeleteEx.Sensitive = true;
			} else {
				btEditEx.Sensitive = false;
				btDeleteEx.Sensitive = false;
			}
				
		}

		

	

		/// <value>
		/// Returns the widget that represents the Person viewer.
		/// </value>
		public Widget Widget{
			get{
				return this;
			}
		}
	}// end of public partial class PersonViewer
}// end of namespace
