
using System;
using Gtk;
using Foodolini.Activities;
using Foodolini.BusinessLogic;

namespace Foodolini.Activities.RegisterExercise
{
	
	[System.ComponentModel.ToolboxItem(true)]
	public partial class ChooseSportsActivity : Gtk.Bin, IActivity
	{
		private NodeStore sportsNode = new NodeStore(typeof(SportsAdapter));
		private IOwner owner;
		
		/// <summary>
		/// Shows all possible Sports Activities
		/// </summary>
		
		public ChooseSportsActivity()
		{
			this.Build();
			this.nvSportsActivity.NodeStore = this.sportsNode;
			this.nvSportsActivity.AppendColumn("Sports Activity", new CellRendererText(), "text", 0);
			
			this.nvSportsActivity.AppendColumn("METs", new CellRendererText(), delegate(TreeViewColumn tree_column, CellRenderer cell, ITreeNode node){
				SportsAdapter n = (SportsAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.METs.ToString();
			});
			
			foreach (SportsActivity spo in SportsActivity.ListActivities()) {
				sportsNode.AddNode(new SportsAdapter(spo));
			}
			
			btEditSportsActivity.Sensitive = false;
			btDeleteSportsActivity.Sensitive = false;
			
			nvSportsActivity.ShowAll();
		}
		
		public void Register(IOwner owner){
			this.owner = owner;
		}
		
		public void Unregister(){
			this.owner = null;			
		}

		/// <summary>
		/// Initiates dialog for creating a new SportsActivity
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtCreateSportsActivityClicked (object sender, System.EventArgs e)
		{
			
			CreateSportsActivityDialog dialog = new CreateSportsActivityDialog();
			ResponseType res = (ResponseType)dialog.Run();
			
			string spName;
			double spMet;
			SportsActivity spSport;
			
			if (res == ResponseType.Ok) {
				spName = dialog.SportsActivityName;
				spMet = dialog.SportsActivityMet;
				spSport = new SportsActivity(spName, spMet);
				spSport.Save();
				this.sportsNode.AddNode(new SportsAdapter(spSport));
				Console.WriteLine("It does");
			}
			
			dialog.Destroy();
			
		}

		/// <summary>
		/// Deletes a SportsActivity
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtDeleteSportsActivityClicked (object sender, System.EventArgs e)
		{
			var sport = (SportsAdapter)this.nvSportsActivity.NodeSelection.SelectedNode;
			if (sport != null){
				ConfirmDialog dialog = new ConfirmDialog();
				
				ResponseType res = (ResponseType)dialog.Run();
		
		        if (res == ResponseType.Ok) {
					this.sportsNode.RemoveNode(sport);
					sport.SportsActivity.Delete();
					if(SportsActivityChanged != null)
						this.SportsActivityChanged(this, new EventArgs());
				}
				dialog.Destroy();	
			}
			
		}

		/// <summary>
		/// Edits a SportsActivity
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtEditSportsActivityClicked (object sender, System.EventArgs e)
		{
			var sport = (SportsAdapter)this.nvSportsActivity.NodeSelection.SelectedNode;
			if (sport.SportsActivity != null){
				
				CreateSportsActivityDialog dialog = new CreateSportsActivityDialog(sport.SportsActivity);
				
				ResponseType res = (ResponseType)dialog.Run();
				
				if (res == ResponseType.Ok) {
					sport.SportsActivity.Name = dialog.SportsActivityName;
					sport.SportsActivity.METS = dialog.SportsActivityMet;
					sport.SportsActivity.Save();
					nvSportsActivity.QueueDraw();
				}
				
				dialog.Destroy();
			}	
			
		}

		protected virtual void OnNvSportsActivityCursorChanged (object sender, System.EventArgs e)
		{
			if(nvSportsActivity.NodeSelection.SelectedNode != null) {
				btEditSportsActivity.Sensitive = true;
				btDeleteSportsActivity.Sensitive = true;
			} else {
				btEditSportsActivity.Sensitive = false;
				btDeleteSportsActivity.Sensitive = false;
			}
			if(SportsActivityChanged != null)
				this.SportsActivityChanged(this, new EventArgs());
		}
		
		/// <value>
		/// Occurs whenever a sportsactivity changes
		/// </value>
		public event EventHandler SportsActivityChanged;
		
		/// <value>
		/// Getter and setter for SportsActivity
		/// </value>
		public SportsActivity SportsActivity {
			get {
				if(this.nvSportsActivity.NodeSelection.SelectedNode != null) {
					return ((SportsAdapter)this.nvSportsActivity.NodeSelection.SelectedNode).SportsActivity;
				}
				return null;
				}
			set {
				foreach (SportsAdapter s in sportsNode) {
					if (s.SportsActivity.Name == value.Name)
						nvSportsActivity.NodeSelection.SelectNode(s);
				}
			}
		}
				
		
		
		public Widget Widget{
			get{
				return this;
			}
		}
	}
}
