using System;
using NUnit.Framework;
using System.IO;
using Foodolini.Database;

/// <summary>
/// Tests of the classes in the Database layer
/// </summary>
namespace Foodolini.Database.Test
{
	/// <summary>
	/// Abstract class for all nUnit tests that tests the Foodolini.Database component
	/// </summary>
	/// <remarks>
	/// This baseclass sets up a Repository with a connection to an empty database prior to every
	/// test in the it's subclass.
	/// </remarks>
	public abstract class DatabaseTest
	{
		/// <summary>
		/// Path to the temporary test database
		/// </summary>
		private const string databasePath = "test.db";
		
		protected Repository repository = null;
		
		/// <summary>
		/// Initializes a Repository connected to an empty database 
		/// </summary>
		[SetUp]
		public void CreateDatabase(){
			//Delete a database if any...
			this.DeleteDatabase();
			this.repository = Repository.ConnectSqlite(databasePath);
		}
		
		/// <summary>
		/// Deletes the test database file, if existing.
		/// </summary>
		[TearDown]
		public void DeleteDatabase(){
			if(this.repository != null)
				this.repository.Dispose();
			this.repository = null;
			if(File.Exists(databasePath))
				File.Delete(databasePath);
		}
	}
}
