
using System;
using NUnit.Framework;
using Foodolini.Database; 

namespace Foodolini.Database.Test
{
	
	/// <summary>
	/// Unit test to the Database layer of FoodDescription
	/// </summary>
	[TestFixture]
	public class FoodDescriptionTest : DatabaseTest 
	{
		/// <summary>
		/// Saves a FoodDescription to the database and test if the object recives an ID. 
		///	Load the FoodDescription from the database and test if the data match
		/// </summary>
		/// <remarks>
		/// Fiext error in database handling of TimeSpan 
		/// </remarks>
		[Test]
		public void SaveloadTest()
		{
			FoodDescription food;
			food = new FoodDescription();
			food.CommercialName = "ymer";
			food.ExpirationAfterOpening  = new TimeSpan(10,20,30, 0 ,40).Ticks;
			food.FoodGroupId = 44;
			food.LongDescription = "ymer2";
			food.Manufacturer = "arla";
			food.ShelfLife = new TimeSpan(4,3,2,1).Ticks;
			food.ShortDescription = "short ymer";
			
			this.repository.Add<FoodDescription>(food);
			
			Assert.AreNotEqual (0,food.FoodDescriptionId, "food have been given a database Id"); 
				
			FoodDescription foodload;
			foodload = this.repository.SingleWhere<FoodDescription>("FoodDescriptionId = @0",food.FoodDescriptionId);
			
			TimeSpan expTime = new TimeSpan(foodload.ExpirationAfterOpening.Value);
			Assert.AreEqual(10, expTime.Days ,"test that time stamp match");
			Assert.AreEqual(20, expTime.Hours,"test that time stamp match");
			Assert.AreEqual(30, expTime.Minutes,"test that time stamp match");
			Assert.AreEqual(40, expTime.Milliseconds,"test that time stamp match");
			
			TimeSpan ShelfLife = new TimeSpan(foodload.ShelfLife.Value);
			Assert.AreEqual(4, ShelfLife.Days ,"test that time stamp match");
			Assert.AreEqual(3, ShelfLife.Hours,"test that time stamp match");
			Assert.AreEqual(2, ShelfLife.Minutes,"test that time stamp match");
			Assert.AreEqual(1, ShelfLife.Seconds,"test that time stamp match");
			
			Assert.AreEqual(44,foodload.FoodGroupId);
			Assert.AreEqual("ymer",foodload.CommercialName);
			Assert.AreEqual("ymer2",foodload.LongDescription);
			Assert.AreEqual("arla",foodload.Manufacturer);
			Assert.AreEqual("short ymer",foodload.ShortDescription);
			
		}
	}
}
