
using System;
using NUnit.Framework;
using Foodolini.Database; 

namespace Foodolini.Database.Test
{
	
	/// <summary>
	/// Unit test to the Database layer of FoodItemRowTest
	/// </summary>
	[TestFixture]
	public class FoodItemRowTest : DatabaseTest 
	{
		FoodItemRow food;
		FoodItemRow food2;
		FoodItemRow food3;
		
		
		/// <summary>
		/// add data to food to be used in testing
		/// </summary>
		private void InsertTestData(){
			food = new FoodItemRow();
			food.ConsumedBy = null;
			food.ConsumedDate =  null; 
			food.ExpirationDate   =  System.DateTime.Now.AddDays(5);
			food.FoodDescriptionId = 5;
			food.IsOpen = false;
			food.Quantity = 7;
			
			food2 = new FoodItemRow();
			food2.ConsumedBy = 1;
			food2.ConsumedDate =  System.DateTime.Now.AddDays(-1)   ; 
			food2.ExpirationDate   =  System.DateTime.Now.AddDays(3);
			food2.FoodDescriptionId = 9;
			food2.IsOpen = false ;
			food2.Quantity = 3;
			
			food3 = new FoodItemRow();
			food3.ConsumedBy = 2;
			food3.ConsumedDate =  System.DateTime.Now.AddDays(-1)   ; 
			food3.ExpirationDate   =  System.DateTime.Now.AddDays(3);
			food3.FoodDescriptionId = 4;
			food3.IsOpen = true ;
			food3.Quantity = 5;
		}
		/// <summary>
		/// Saves a FoodItemRow to the database and test if the object recives an ID. 
		///	Load the FoodItemRow from the database and test if the data match
		/// </summary>
		[Test]
		public void SaveLoadTest()
		{
			InsertTestData();
			
			this.repository.Add<FoodItemRow>(food);
			Assert.AreNotEqual(0,food.FoodItemRowId," check that rowId has been set");
			this.repository.Add<FoodItemRow>(food2);
			Assert.AreNotEqual(0,food2.FoodItemRowId," check that rowId has been set");
			
			this.repository.Add<FoodItemRow>(food3);
			Assert.AreNotEqual(0,food3.FoodItemRowId," check that rowId has been set");
			
			foreach (FoodItemRow row in this.repository.All<FoodItemRow>()){
				if ( row.ConsumedBy == 2) {
					Assert.AreEqual(true ,row.IsOpen,"check IsOpen is loaded 2");
					Assert.AreEqual(2 ,row.ConsumedBy,"check ConsumedBy is loaded 2"); 
					Assert.AreEqual(System.DateTime.Now.AddDays(-1).Day  ,row.ConsumedDate.Value.Day,"check ConsumedDate is loaded 2");  
					Assert.AreEqual(System.DateTime.Now.AddDays(3).Day  ,row.ExpirationDate.Day,"check ExpirationDate is loaded 2");  
					Assert.AreEqual(4 ,row.FoodDescriptionId,"check FoodDescriptionId is loaded 2");  
					Assert.AreEqual(5 ,row.Quantity,"check Quantity is loaded 2");  
				}else if( row.ConsumedBy == 1) {
					Assert.AreEqual(false ,row.IsOpen,"check IsOpen is loaded 1");  
					Assert.AreEqual(1 ,row.ConsumedBy,"check ConsumedBy is loaded 1"); 
					Assert.AreEqual(System.DateTime.Now.AddDays(-1).Day  ,row.ConsumedDate.Value.Day ,"check ConsumedDate is loaded 1");  
					Assert.AreEqual(System.DateTime.Now.AddDays(3).Day  ,row.ExpirationDate.Day,"chek ExpirationDate is loaded 1");  
					Assert.AreEqual(9 ,row.FoodDescriptionId,"check FoodDescriptionId is loaded 1");  
					Assert.AreEqual(3 ,row.Quantity,"chck Quantity is loaded 1");  
				}else{
					Assert.AreEqual(false ,row.IsOpen,"check IsOpen is loaded null");  
					Assert.AreEqual(null ,row.ConsumedBy,"check ConsumedBy is loaded null"); 
					Assert.AreEqual(null ,row.ConsumedDate,"check ConsumedDate is loaded null");  
					Assert.AreEqual(System.DateTime.Now.AddDays(5).Day  ,row.ExpirationDate.Day,"chek ExpirationDate is loaded null");  
					Assert.AreEqual(5 ,row.FoodDescriptionId,"check FoodDescriptionId is loaded null");  
					Assert.AreEqual(7 ,row.Quantity,"chck Quantity is loaded null");  
				}
					
			}
		}
		
		/// <summary>
		/// inset a FoodItemRow to the database and delets it again while checking that 
		/// the correct data were deleted.
		/// </summary>
		/// <remarks>
		/// Fiext crash error if a item were to be delete multible times.
		/// </remarks>
		[Test]
		public void DeleteTest()
		{
			int counter = 0;
			
			InsertTestData();
			this.repository.Add<FoodItemRow>(food); //quantity 7
			this.repository.Add<FoodItemRow>(food2); //quantity 3
			this.repository.Add<FoodItemRow>(food3); //quantity 5
			
			foreach (FoodItemRow i in this.repository.All<FoodItemRow>()){
				counter ++;
			}
			Assert.AreEqual(3,counter,"check number of rows");
			
			
			this.repository.Delete<FoodItemRow>(food2);
			counter = 0;
			foreach (FoodItemRow i in this.repository.All<FoodItemRow>()){
				counter ++;
				Assert.AreNotEqual(3,i.Quantity ,"check that the right one have been deleted");		
			}
			Assert.AreEqual(2,counter,"check number of rows");		
			
			
			this.repository.Delete<FoodItemRow>(food);
			counter = 0;
			foreach (FoodItemRow i in this.repository.All<FoodItemRow>()){
				counter ++;
				Assert.AreNotEqual(7,i.Quantity ,"check that the right one have been deleted");		
			}
			Assert.AreEqual(1,counter,"check number of rows");	
			
			
			this.repository.Delete<FoodItemRow>(food);
			counter = 0;
			foreach (FoodItemRow i in this.repository.All<FoodItemRow>()){
				counter ++;
				Assert.AreEqual(5,i.Quantity ,"check that only food3 remains");		
			}
			Assert.AreEqual(1,counter,"check number of rows");	
		}
		
		/// <summary>
		/// test the database update function to update an existing FoodItemRow by saving data to the 
		/// database, then update it and check that the data is correct.
		/// </summary>
		[Test]
		public void UpdateTest()
		{
			InsertTestData();
			this.repository.Add<FoodItemRow>(food); //quantity 7
			this.repository.Add<FoodItemRow>(food2); //quantity 3
			this.repository.Add<FoodItemRow>(food3); //quantity 5
			
			food.IsOpen = true;
			food.Quantity = 44; 
			food3.ConsumedBy = 22;
			
			this.repository.Update<FoodItemRow>(food);
			this.repository.Update<FoodItemRow>(food3);
			this.repository.Update<FoodItemRow>(food2);	
			
			int counter = 0;
			foreach (FoodItemRow i in this.repository.Where<FoodItemRow>("FoodDescriptionId = @0", 5)){
				counter++;
				Assert.AreEqual(44,i.Quantity ,"seach must match");
				Assert.AreEqual(true,i.IsOpen  ,"isopen must match");
			}
			Assert.AreEqual(1,counter,"only one row selected");
			
			
			counter = 0;
			foreach (FoodItemRow i in this.repository.Where<FoodItemRow>("Quantity = @0", 3)){
				counter++;
				Assert.AreEqual(1,i.ConsumedBy ,"seach must match");
			}
			Assert.AreEqual(1,counter,"only one row selected");		
			
			
			counter = 0;
			foreach (FoodItemRow i in this.repository.Where<FoodItemRow>("ConsumedBy = @0", 22)){
				counter++;
				Assert.AreEqual(4,i.FoodDescriptionId ,"seach must match");
			}
			Assert.AreEqual(1,counter,"only one row selected");		
		}
		
		
		/// <summary>
		/// test the were and singleware command by inserting aditional rows to the database and 
		/// then use the were method to load the data rows specified by the parameters and then test
		/// if the rows match.
		/// </summary>
		[Test]
		public void WereTest()
		{
			InsertTestData();
			this.repository.Add<FoodItemRow>(food);
			this.repository.Add<FoodItemRow>(food2);
			food2.Quantity = 4;
			this.repository.Add<FoodItemRow>(food2);
			food2.Quantity = 5;
			this.repository.Add<FoodItemRow>(food2);
			int counter = 0;
			foreach (FoodItemRow i in this.repository.Where<FoodItemRow>("FoodDescriptionId = @0", 9)){
				counter++;
				Assert.AreEqual(1,i.ConsumedBy,"seach must match");
			}
			Assert.AreEqual(3,counter,"found all 1 ");
			foreach (FoodItemRow i in this.repository.Where<FoodItemRow>("FoodDescriptionId = @0 AND Quantity = @1", 5,7)){
				Assert.AreEqual(null,i.ConsumedBy,"seach must match");	
			}
			counter = 0;
			foreach (FoodItemRow i in this.repository.Where<FoodItemRow>("FoodDescriptionId = @0 AND IsOpen = @1 AND ConsumedBy = @2", 9,false,1)){
				counter++;
				Assert.AreEqual(1,i.ConsumedBy,"seach must match");
			}
			Assert.AreEqual(3,counter,"found all 2");
			
			FoodItemRow testFood;
			testFood=  this.repository.SingleWhere<FoodItemRow >("Quantity = @0",4);
				Assert.AreEqual(1,testFood.ConsumedBy, "single seach must match");
			
				
		}
	}
}
