using System;
using NUnit.Framework;
using Foodolini.BusinessLogic;
using System.IO;

/// <summary>
/// Tests of classes in the business logic layer
/// </summary>
namespace Foodolini.BusinessLogic.Test
{
	/// <summary>
	/// Abstract class for all nUnit tests that tests the Foodolini.BusinessLogic component
	/// </summary>
	/// <remarks>
	/// At SetUp this initializes Settings.Instance to a test database with commit disabled.
	/// At TearDown it disposes Settings.Instance
	/// </remarks>
	public abstract class LogicTest
	{
		/// <summary>
		/// Creates the database used in the logic tests.
		/// </summary>
		[SetUp]
		public void SetCreateTestDatabase(){
			//Reinitialize all static types
			foreach(Type T in typeof(Settings).Assembly.GetTypes())
				if(T.TypeInitializer != null)
					T.TypeInitializer.Invoke(null ,new object[0]);
			Settings.Instance.OpenSqliteDatabase("foodolini.db3", true);
		}
		
		/// <summary>
		/// Delete the database used in the logic tests.
		/// </summary>
		[TearDown]
		public void DeleteTestDatabase(){
			Settings.Instance.Dispose();
		}
	}
}
