
using System;
using Foodolini.Activities;
using Foodolini.BusinessLogic;
using Gtk;
using System.Collections.Generic;

namespace Foodolini.Activities.Cookbook
{
	/// <summary>
	/// An activity where the users how attended a meal can be selected.
	/// </summary>
	[FoodoliniActivity("Select users", true)]
	public partial class UserSelector : Gtk.Dialog, IActivity
	{
		
		protected NodeStore userStorage = new NodeStore(typeof(PersonAdapter));
		protected NodeStore userSelectedStorage = new NodeStore(typeof(PersonAdapter));
		private Dictionary<Person, double> users = new Dictionary<Person, double>();
		
		public UserSelector()
		{
			this.Build();
			
			//Set up nodeview
			this.nvUsers.NodeStore = userStorage;
			this.nvUsers.AppendColumn("Users", new CellRendererText(), (tree_colum, cell, node)=>{
				PersonAdapter n = (PersonAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Name;
			});
			
			this.nvSelectedUsers.NodeStore = userSelectedStorage;
			this.nvSelectedUsers.AppendColumn("Users", new CellRendererText(), (tree_colum, cell, node)=>{
				PersonAdapter n = (PersonAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.Name;
			});
			this.nvSelectedUsers.AppendColumn("Percentage eaten", new CellRendererText(), (tree_colum, cell, node)=>{
				PersonAdapter n = (PersonAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.PercentageEaten.ToString();
			});
			
			bxUnselected.ShowAll();
			bxSelected.ShowAll();
			
			//Adds all useres to the userStorage
			foreach (var person in Person.ListUsers())
				userStorage.AddNode(new PersonAdapter(person));
			
			this.buttonOk.Sensitive = false;
			
		}
		
		/// <summary>
		/// Checks if any users are selected. If not the Ok buttons sensitivty is set to false
		/// </summary>
		private void checkSelectedPeople()
		{
			this.buttonOk.Sensitive = false;
			if(this.userSelectedStorage.GetEnumerator().MoveNext())
				this.buttonOk.Sensitive = true;
		}
		
		/// <summary>
		/// Returns all users
		/// </summary>
		/// <returns>
		/// A <see cref="Dictionary"/>
		/// </returns>
		public Dictionary<Person, double> GetUsers ()
		{
			foreach (PersonAdapter user in userSelectedStorage)
				users.Add(user.Person, user.PercentageEaten);
			return users;
		}

		/// <summary>
		/// Adds a user to the userSelectedStorage
		/// </summary>
		protected virtual void OnAddUserClick (object sender, System.EventArgs e)
		{
			if(nvUsers.NodeSelection.SelectedNode != null)
			{
				PersonAdapter pa = ((PersonAdapter)nvUsers.NodeSelection.SelectedNode);
				pa.PercentageEaten = sbProcentageEaten.Value;
				
				userSelectedStorage.AddNode(pa);
				checkSelectedPeople();
			}
		}

		/// <summary>
		/// Removes a user form the userSelectedStorage
		/// </summary>
		protected virtual void OnBtnRemoveUserClicked (object sender, System.EventArgs e)
		{
			if(nvSelectedUsers.NodeSelection.SelectedNode != null)
			{
				userSelectedStorage.RemoveNode((PersonAdapter)nvSelectedUsers.NodeSelection.SelectedNode);
				checkSelectedPeople();
			}
		}
		
		#region IActivity
		public void Register(IOwner owner){
			//We don't owner to just forget about it...
		}
		
		public void Unregister(){
			//Don't have any ties to owner to forget
		}
		
		public Widget Widget{
			get{
				return this;
			}
		}
		#endregion
	}
}
