using Gtk;
using Gdk;
using System;
using Foodolini.Activities;
using Foodolini.BusinessLogic;
using System.Collections.Generic;

namespace Foodolini.Activities.Home
{
	[FoodoliniActivity("Home", false, "Home.home.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class HomeActivity : Gtk.Bin, IActivity
	{

		public HomeActivity()
		{
			//Build the UI
			this.Build();
			
			//Style the eventbox
			this.styleFacilitator.ModifyBg(StateType.Normal, new Color(0xff, 0xff, 0xff));
			this.Shown += delegate {
				using(Pixbuf img = Pixbuf.LoadFromResource("Home.bg.png")){
					Pixmap pix, mask;
					img.RenderPixmapAndMaskForColormap(this.styleFacilitator.Colormap, out pix, out mask, 0);
					this.styleFacilitator.Style.SetBgPixmap(StateType.Normal, pix);
					pix.Dispose();
					mask.Dispose();
				}
			};
			
			//Populate inventory view
			this.PopulateInventoryView();
		}
		
		#region IActivity implementation
		
		private IOwner owner = null;
		
		public void Register(IOwner owner){
			this.owner = owner;
		}
		
		public void Unregister(){
			this.owner = null;
		}
		
		public Widget Widget{
			get{return this;}
		}
		
		#endregion
		
		#region Inventory view
		
		private NodeStore store;
		
		private void PopulateInventoryView(){
			store = new NodeStore(typeof(FoodItemAdapter));
			
			//Sort the FoodItems
			List<FoodItem> items = new List<FoodItem>(FoodItem.ListFoodItems());
			items.Sort((f1, f2) => {
				if(f1.ExpirationDate < f2.ExpirationDate)
					return -1;
				if(f1.ExpirationDate == f2.ExpirationDate)
					return 0;
				return 1;
			});
			
			//Add fooditems to store
			foreach(var item in items)
				store.AddNode (new FoodItemAdapter(item));
			
			this.InventoryView.NodeStore = store;

			//Append cols
			this.InventoryView.AppendColumn ("Food", new CellRendererText (), "text", 0);
			this.InventoryView.AppendColumn ("Quantity", new CellRendererText (), "text", 1);
			this.InventoryView.AppendColumn ("Expiration date", new CellRendererText (), (column, cell, node) =>
			{
				FoodItemAdapter n = (FoodItemAdapter)node;
				CellRendererText c = (CellRendererText)cell;
				c.Text = n.ExpirationDate.ToShortDateString();
			});
			this.InventoryView.AppendColumn ("Open/Closed", new CellRendererText(), "text",  3);
		}

		#endregion

		protected virtual void OnInventoryButtonClicked (object sender, System.EventArgs e){
			this.owner.LoadActivity("Inventory");
		}

		protected virtual void OnRegisterFoodItemButtonClicked (object sender, System.EventArgs e){
			this.owner.LoadActivity("Food registration");
		}

		protected virtual void OnCookbookButtonClicked (object sender, System.EventArgs e){
			this.owner.LoadActivity("Cookbook");
		}

		protected virtual void OnNewRecipeButtonClicked (object sender, System.EventArgs e){
			this.owner.LoadActivity("Create recipe");
		}

		protected virtual void OnIngredientsButtonClicked (object sender, System.EventArgs e){
			this.owner.LoadActivity("Ingredients");
		}

		protected virtual void OnProfileButtonClicked (object sender, System.EventArgs e){
			this.owner.LoadActivity("User profile");
		}

		protected virtual void OnAdminButtonClicked (object sender, System.EventArgs e){
			this.owner.LoadActivity("User administration");
		}

		protected virtual void OnInventoryViewRowActivated (object o, Gtk.RowActivatedArgs args){
			this.owner.LoadActivity("Inventory");
		}
	}
}
