
using System;
using Gtk;
using Foodolini.Activities;
using Foodolini.BusinessLogic;

namespace Foodolini.Activities.Ingredients
{
	/// <summary>
	/// A TreeNode adapter for the <see cref="Ingredient"/> class. 
	/// </summary>
	[TreeNode(ListOnly = true)]
	public class IngredientAdapter : TreeNode
	{
		/// <summary>
		/// The <see cref="Ingredient"/> instance.  
		/// </summary>
		public Ingredient Ingredient { get; private set; }
		
		/// <summary>
		/// Instantiates a new IngredientAdapter with the supplied <see cref="Ingredient"/>.
		/// </summary>
		/// <param name="ingredient">
		/// A <see cref="Ingredient"/>
		/// </param>
		public IngredientAdapter (Ingredient ingredient)
		{
			this.Ingredient = ingredient;
		}

		/// <summary>
		/// The name of the ingredient. 
		/// </summary>
		[Gtk.TreeNodeValue(Column = 0)]
		public string Name {
			get { return this.Ingredient.LongDescription; }
		}

		/// <summary>
		/// The amount of protein/100g in the ingredient. 
		/// </summary>
		[Gtk.TreeNodeValue(Column = 1)]
		public double Protein {
            get { return this.Ingredient.Protein; }
		}
		
		/// <summary>
		/// The amount of carbohydrates/100g in the ingredient. 
		/// </summary>
		[Gtk.TreeNodeValue(Column = 2)]
		public double Carbohydrates {
            get { return this.Ingredient.Carbohydrates; }
		}
		
		/// <summary>
		/// The amount of fat/100g in the ingredient. 
		/// </summary>
		[Gtk.TreeNodeValue(Column = 3)]
		public double Fat {
            get { return this.Ingredient.Fat; }
		}

	}
}
