
using System;
using Gtk;
using Foodolini.Activities;
using Foodolini.BusinessLogic;


namespace Foodolini.Activities.RegisterExercise
{
	[FoodoliniActivity("Edit exercise", true)]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class ExerciseEditorDialog : Gtk.Dialog, IActivity
	{
		private IOwner owner;
		private Exercise exercise;
		private ChooseSportsActivity cs = new ChooseSportsActivity();
		
		public ExerciseEditorDialog(Exercise chosenExercise)
		{
			this.Build();
			this.exercise = chosenExercise;
			spbtHours.Value = (double)exercise.Duration.Hours;
			spbtMinutes.Value = (double)exercise.Duration.Minutes;
			calDatePerformed.Date = exercise.PerformedDate;
			vbChooseSport.Add(cs);
			cs.SportsActivity = exercise.Activity;
			cs.Show();
			cs.SportsActivityChanged += HandleSportsActivityChanged;
			this.buttonOK.Sensitive = this.cs.SportsActivity != null;;
		}

		void HandleSportsActivityChanged(object sender, EventArgs e){
			this.buttonOK.Sensitive = this.cs.SportsActivity != null;
		}
		
		protected virtual void OnBtSaveExerciseClicked (object sender, System.EventArgs e)
		{
		}
		
		/// <summary>
		/// Saves the edited exercise
		/// </summary>
		public void SaveExercise()
		{
			TimeSpan time = new TimeSpan((int)spbtHours.Value, (int)spbtMinutes.Value, 0);
			
			
			this.exercise.Activity = this.cs.SportsActivity;
			this.exercise.Duration = time;
			this.exercise.PerformedDate = calDatePerformed.Date;
			this.exercise.Save();
		}

		/// <value>
		/// Getter for the edited exercise
		/// </value>
		public Exercise Exercise{
			get {return this.exercise;}
		}
		
		public void Register(IOwner owner){
			this.owner = owner;
		}
		
		public void Unregister(){
			
			this.owner = null;
		}

		protected virtual void OnButtonOKClicked (object sender, System.EventArgs e)
		{
		}

		protected virtual void OnButtonCancelClicked (object sender, System.EventArgs e)
		{
		}
		public Widget Widget{
			get{
				return this;
			}
		}
		
	}
}
