
using System;
using Gtk;
using Foodolini.Activities;
using Foodolini.BusinessLogic;

namespace Foodolini.Activities.RegisterExercise
{	
	[FoodoliniActivity("Register exercise", true,"Exercise.png")]
	[System.ComponentModel.ToolboxItem(true)]
	public partial class RegisterExerciseDialog : Gtk.Dialog, IActivity
	{
		private IOwner owner;
		private Exercise exercise;
		private ChooseSportsActivity cs = new ChooseSportsActivity();
		
		public RegisterExerciseDialog()
		{
			this.Build();
			this.Shown += HandleShown;
			vbChooseSport.Add(cs);
			cs.Show();
			cs.SportsActivityChanged += HandleSportsActivityChanged;
			this.btSaveExercise.Sensitive = false;
		}

		void HandleSportsActivityChanged(object sender, EventArgs e){
			this.btSaveExercise.Sensitive = this.cs.SportsActivity != null;
		}
		
		void HandleShown(object sender, EventArgs e)
		{
			CheckUser();
		}
		
		public void Register(IOwner owner){
			this.owner = owner;
			this.owner.CurrentUserChanged += HandleCurrentUserChanged;
		}

		void HandleCurrentUserChanged(object sender, PersonEventArgs e)
		{
			CheckUser();
		}
		
		/// <summary>
		/// Checkuser greys the "Save exercise" button if the user is Anonymous
		/// and makes it active if a user is registered
		/// </summary>
		private void CheckUser() {
			if (this.owner.CurrentUser == null) {
				btSaveExercise.Sensitive = false;
			} else {
				btSaveExercise.Sensitive = true;
			}
		}
		
		/// <summary>
		/// Registers if the user "logs out"
		/// </summary>
		public void Unregister(){
			this.owner.CurrentUserChanged -= HandleCurrentUserChanged;
			this.owner = null;
		}

		/// <summary>
		/// Saves the Exercise entered by the user on the Person
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtSaveExerciseClicked (object sender, System.EventArgs e)
		{
			TimeSpan time = new TimeSpan((int)spbtHours.Value, (int)spbtMinutes.Value, 0);
			this.exercise = new Exercise(this.cs.SportsActivity, time, this.owner.CurrentUser, calDatePerformed.Date);
			this.exercise.Save();
			
		}

		/// <summary>
		/// Cancels the registration of the exercise
		/// </summary>
		/// <param name="sender">
		/// A <see cref="System.Object"/>
		/// </param>
		/// <param name="e">
		/// A <see cref="System.EventArgs"/>
		/// </param>
		protected virtual void OnBtCancelClicked (object sender, System.EventArgs e)
		{
		}
		
		/// <value>
		/// Getter for the created Exercise
		/// </value>
		public Exercise Exercise{
			get {return this.exercise;}
		}
		
		public Widget Widget{
			get{
				return this;
			}
		}
	}
	
	
}
