using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;


namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// A diet.
    /// </summary>
    public class Diet
    {
        private DietRow dietRow;
        private bool modified = true;

        /// <summary>
        /// Loads a diet from the supplied diet id.
        /// </summary>
        /// <param name="dietId"></param>
        internal Diet(long dietId)
        {
            this.modified = false;
            throw new NotImplementedException();
        }

        public Diet(string name, double proteinPercentage,
            double carbPercentage, double fatPercentage, double calorieFactor)
        {
            this.dietRow = new DietRow();

            this.Name = name;
            this.ProteinPercentage = proteinPercentage;
            this.CarbPercentage = carbPercentage;
            this.FatPercentage = fatPercentage;
            this.CalorieFactor = calorieFactor;
        }


        private static Diet defaultDiet = null;
        public static Diet DefaultDiet()
        {
            // TODO: Load from database instead!
            if (defaultDiet == null)
                defaultDiet = new Diet("Default Diet", 0.33, 0.33, 0.33, 0);
            return defaultDiet;
        }

        /// <summary>
        /// Id of the diet.
        /// </summary>
        internal long DietId
        {
            get
            {
                return this.dietRow.DietId;
            }
        }

        /// <summary>
        /// The name of the diet.
        /// </summary>
        public string Name
        {
            get
            {
                return this.dietRow.Title;
            }
            set
            {
                this.dietRow.Title = value;
                modified = true;
            }
        }

        /// <summary>
        /// Protein percentage of the diet.
        /// </summary>
        public double ProteinPercentage
        {
            get
            {
                return this.dietRow.ProteinPercentage;
            }
            set
            {
                this.dietRow.ProteinPercentage = value;
                modified = true;
            }
        }

        /// <summary>
        /// Carbohydrate percentage of the diet.
        /// </summary>
        public double CarbPercentage
        {
            get
            {
                return this.dietRow.CarbPercentage;
            }
            set
            {
                this.dietRow.CarbPercentage = value;
                modified = true;
            }
        }

        /// <summary>
        /// Fat percentage of the diet.
        /// </summary>
        public double FatPercentage
        {
            get
            {
                return this.dietRow.FatPercentage;
            }
            set
            {
                this.dietRow.FatPercentage = value;
                modified = true;
            }
        }

        /// <summary>
        /// Factor to add to the baseline caloric needs of a person.
        /// </summary>
        public double CalorieFactor
        {
            get { return this.dietRow.CalorieFactor; }
            set
            {
                this.dietRow.CalorieFactor = value;
                modified = true;
            }
        }

        /// <summary>
        /// Saves changes to the database.
        /// </summary>
        public void Save()
        {
            if (DietId == 0)
                Settings.Repo.Add(this.dietRow);
            else if (modified)
                Settings.Repo.Update(this.dietRow);
        }

        /// <summary>
        /// Deletes a diet permanently.
        /// </summary>
        public void Delete()
        {
            if (this.dietRow.DietId != 0)
                Settings.Repo.Delete(this.dietRow);
            this.modified = true;
        }

        /// <summary>
        /// Calculates the total daily calorie needs of a person
        /// based on the activity factor and the goal of the diet.
        /// </summary>
        /// <param name="person"></param>
        /// <returns></returns>
        public int GetRequiredCalories(Person person)
        {
            return Convert.ToInt32(person.BMR * person.ActivityFactor + (person.BMR * this.CalorieFactor));
        }
    }
}

