using System;
using System.Collections.Generic;
using System.Text;
using Foodolini.Database;

namespace Foodolini.BusinessLogic
{
    /// <summary>
    /// An exercise performed by a person.
    /// </summary>
    public class Exercise
    {
        private ExerciseRow row;
		private bool modified = true;

        public Exercise(SportsActivity activity, TimeSpan duration, Person performedBy, DateTime performedDate){
			this.row = new ExerciseRow();
            this.activity = activity;
            this.Duration = duration;
            this.PerformedBy = performedBy;
            this.PerformedDate = performedDate;
			this.modified = true;
        }
		
		internal Exercise(ExerciseRow row){
			this.row = row;
			this.modified = false;
		}

        private SportsActivity activity;

        /// <summary>
        /// The sports activity performed.
        /// </summary>
        public SportsActivity Activity
        {
            get {
				try{
	                if(this.activity == null)
	                    this.activity = new SportsActivity(this.row.SportsActivityId);
                	return this.activity;
				}
				catch(System.Exception e){
					return null;
				}
            }
            set { 
				this.activity = value;
				this.modified = true;
			}
        }

        /// <summary>
        /// The duration of the exercise.
        /// </summary>
        public TimeSpan Duration
        {
            get { return new TimeSpan(this.row.Duration); }
            set { 
				this.modified = true;
				this.row.Duration = value.Ticks;
			}
        }

        /// <summary>
        /// The date for when the exercise was performed.
        /// </summary>
        public DateTime PerformedDate
        {
            get { return this.row.PerformedDate; }
            set {
                this.modified = true;
                this.row.PerformedDate = value;
            }
        }

        /// <summary>
        /// The person who performed the exercise.
        /// </summary>
        public Person PerformedBy
        {
            get {
                return Person.GetById(this.row.UserId);
            }
            set { 
				this.modified = true;
				this.row.UserId = value.Id; 
			}
        }

        /// <summary>
        /// Saves changes to the database.
        /// </summary>
        public void Save(){
			if(this.activity != null){
				this.activity.Save();
				if(this.row.SportsActivityId != this.activity.Id){
					this.row.SportsActivityId = this.activity.Id;
					this.modified = true;
				}
			}
            if (this.row.ExerciseRowId == 0)
                Settings.Repo.Add(this.row);
            else{
				if(this.modified)
                	Settings.Repo.Update(this.row);
			}
			
			this.modified = false;
        }

        /// <summary>
        /// Permanently deletes the exercise.
        /// </summary>
        public void Delete()
        {
            if (this.row.ExerciseRowId != 0)
                Settings.Repo.Delete(this.row);
            this.row.ExerciseRowId = 0;
			this.modified = true;
        }
		
		public static IEnumerable<Exercise> ListExercises(){
			foreach(var exer in Settings.Repo.All<ExerciseRow>())
				yield return new Exercise(exer);
		}

    }
}
