
using System;
using NUnit.Framework;

namespace Foodolini.Database.Test
{
	/// <summary>
	/// Unit test to the Database layer of User
	/// </summary>
	[TestFixture]
	public class UserTest : DatabaseTest
	{
		[Test]
		public void CreateTable(){
			this.repository.CreateTable<User>();
		}
		
		/// <summary>
		/// Saves a User to the database and test if the object recives an ID. 
		/// </summary>
		[Test]
		public void Save(){
			User user = new User();
			user.BirthDate = new DateTime(1988, 1, 9);
			user.DietId = null;
			user.UserName = "Jopsen";
			user.FullName = "Jonas Finnemann Jensen";
			user.Gender = 1;
			user.Height = 1.79;
			user.Weight = 78;
			this.repository.Add<User>(user);
			
			Assert.AreEqual(1, user.UserId);
		}
		
		/// <summary>
		/// Saves a User to the database and test if the object recives an ID. 		
		/// Load the User from the database and test if the data match
		/// </summary>
		[Test]
		public void SaveAndLoad(){
			User user = new User();
			user.BirthDate = new DateTime(1988, 1, 9);
			user.DietId = null;
			user.UserName = "Jopsen";
			user.FullName = "Jonas Finnemann Jensen";
			user.Gender = 5;
			user.Height = 1.79;
			user.Weight = 78;

			this.repository.Add<User>(user);
			
			user.UserName = "Karsten";
			user.FullName = "Karsten Jacobsen";
			this.repository.Add<User>(user);
			
			int reg = 0;
			foreach(var u in this.repository.Query<User>("SELECT * from USERS")){ 
				if(u.UserName == "Karsten" || u.UserName == "Jopsen")
					reg++;
			}
			Assert.AreEqual(2, reg);
		}
	}
}
