
using System;
using NUnit.Framework;
using Foodolini.BusinessLogic;
using System.Collections.Generic;

namespace Foodolini.BusinessLogic.Test
{

	/// <summary>
	/// Unit test to the Model layer of Ingredient
	/// </summary>
	[TestFixture]
	public class IngredientTest : LogicTest
	{
		/// <summary>
		/// Try to Create an Ingredient and save it
		/// </summary>
		[Test]
		public void CreateIngredientTest()
		{
			Ingredient ingredient = new Ingredient("Parmesan Cheese");
			ingredient.Save();
		}
		
		/// <summary>
		/// Test that the Ingredient search function works by making some searches and check that 
		/// the database returns the correct Ingredients. 
		/// </summary>
		[Test]
		public void SearchTest()
		{
			List<Ingredient> ingredient_list1 = new List<Ingredient>(Ingredient.ListByCategory("Dairy and Egg Products"));
			Assert.AreEqual(236, ingredient_list1.Count);
			
			List<Ingredient> ingredient_list2 = new List<Ingredient>(Ingredient.ListByCategory("Dairy and Egg Products", "Cheese, colby"));
			Assert.AreEqual(1, ingredient_list2.Count);
			
			List<Ingredient> ingredient_list3 = new List<Ingredient>(Ingredient.ListByCategory("Dairy and Egg Products", "Cheese, kolby"));
			Assert.IsEmpty(ingredient_list3);
			
			IList<Nutrient> nut_list = new List<Nutrient>(Nutrient.ListNutritionDefinitions());
			ingredient_list2[0].Nutrients.Add(nut_list[0], 23);
			Assert.AreEqual(true, ingredient_list2[0].Nutrients.ContainsKey(nut_list[0]));
			Assert.AreEqual(true, ingredient_list2[0].Nutrients.Contains(new KeyValuePair<Nutrient, double>(nut_list[0], 23)));
			
			Assert.AreEqual(false, ingredient_list1[0].Nutrients.Contains(new KeyValuePair<Nutrient, double>(nut_list[0],23)));
			
		}
	}
}
