using System;
using NUnit.Framework;
using Foodolini.BusinessLogic;
using Foodolini.Database;

namespace Foodolini.BusinessLogic.Test
{
	/// <summary>
	/// Unit test to the Model layer of Person
	/// </summary>
	[TestFixture]
	public class PersonTest : LogicTest
	{
		/// <summary>
		/// Try to create a person
		/// </summary>
		[Test]
		public void CreatePersonTest (){
			Person per = new Person ("Per", "Postmand Per", new DateTime (1947, 5, 2), Gender.Male, 180, 70, 1.5);
			per.Save ();
			Person heidi = new Person ("Heidi", "Frejdi Heidi", new DateTime (1968, 2, 21), Gender.Female, 173, 58, 4.7);
			heidi.Save();
			Person janus = new Person ("Janus", "Jetjager Janus", new DateTime (1978, 11, 21), Gender.Male, 190, 89, 2.1);
			janus.Save();
		}
		/// <summary>
		/// Test that a Person can be saved and loaded by saving a new Person and then load it again to check 
		/// if the loaded data match.
		/// </summary>
		[Test]
		public void LoadCreatedPersonTest (){
			this.CreatePersonTest ();
			foreach (var p in Person.ListUsers ()) {
				if (p.UserName == "Per")
					Assert.AreEqual (p.BirthDate, new DateTime (1947, 5, 2));
				}
		}
		/// <summary>
		/// Test that a Person can be loaded.
		/// </summary>
		[Test]
		public void LoadPersonTest (){
			foreach (var p in Person.ListUsers ()) {
				if (p.UserName == "jopsen") {
					Assert.AreEqual (p.FullName, "Jonas Finnemann Jensen");
					Assert.AreEqual (p.BirthDate, new DateTime (1988, 1, 9));
				}
			}
		}
		/// <summary>
		/// Tests that a Person can be deleted. by first creading a Person and the delete it again an test
		/// that the Person have been deleted.
		/// </summary>
		[Test]
		public void DeletePersonTest(){
			this.CreatePersonTest();
			Person pip = null;	
			foreach (Person p in Person.ListUsers()){
				if (p.FullName == "Frejdi Heidi"){
					pip = p; //Pip is Heidi if Heidi is in the list 
				}
			}
			pip.Delete(); //Deletes Heidi as pip is Heidi.
			
			foreach (Person p in Person.ListUsers()){
				if (p.FullName == "Fredi Heidi"){
					Assert.Fail(); //If Heidi is in the list the test must fail. 
				}
				if (p.UserName == "Per"){
					Assert.AreEqual (p.UserName, "Per");
				}
				if (p.UserName == "Janus")
					Assert.AreEqual (p.UserName, "Janus");
			}
		}
		/// <summary>
		/// Tests that a persons details can be changed.
		/// Here changed the gender from male to female, the age 10 years younger, the height. 
		/// Changes also affect the BMR and MET.
		/// </summary>
		/// <remarks>
		/// Revealed BMR in integers and METS in doubles. Changed the METS to integers and made a cast in the formula.
		/// </remarks>
		[Test]
		public void EditPersonTest(){
			this.CreatePersonTest();
			foreach (Person p in Person.ListUsers()){
				if(p.FullName == "Postmand Per"){
					p.BirthDate = new DateTime (1957, 11, 23); //Then he is 52 years
					p.Gender = Gender.Female;
					p.Height = 230;
					p.Weight = 80;
					p.ActivityFactor = 9;
				}
			}
			foreach (Person p in Person.ListUsers()){
				if (p.UserName == "Per"){
					Assert.AreEqual(52, p.Age, "Expected 52 years (This is only expected age until 22 november 2010)");
					Assert.AreEqual(Gender.Female, p.Gender,"Expected female");
					Assert.AreEqual(230, p.Height, "Expected 230");
					Assert.AreEqual(80, p.Weight, "Expected 80 kilo");
					Assert.AreEqual(9, p.ActivityFactor, "Expected 9");
					Assert.AreEqual(1816, p.BMR, "Expected 1816"); //Flooring the integers.
					Assert.AreEqual(90, p.METS, "Expected 90 from flooring");
				}
			}
		}
	}
}

