
using System;
using NUnit.Framework;
using Foodolini.BusinessLogic;
using System.Collections.Generic;

namespace Foodolini.BusinessLogic.Test
{

	/// <summary>
	/// Unit test to the Model layer of Recipe
	/// </summary>
	[TestFixture]
	public class RecipeTest: LogicTest
 {
		/// <summary>
		/// Try Save an empty Recipee 
		/// </summary>
		[Test]
		public void CreateRecipeTest ()
		{
			Recipe recipe = new Recipe("Sloppy Joe");
			recipe.Save();  
		}	
		
		
		/// <summary>
		/// Try to save a Recipe with Ingrediens and cooking Directions.
		/// </summary>
		[Test]
		public void SaveRecipeTest()
		{
			Recipe recipe = new Recipe("Sloppy Joe");
			recipe.Ingredients.Add(new Ingredient("ingTitle"), 23);
			recipe.Ingredients.Add(new Ingredient("ingName"), 34);
			recipe.Categories.Add("Breakfast");
			recipe.Difficulty = Difficulty.Easy;
			recipe.Directions.Add("Do the first ting");
			recipe.Directions.Add("Do the next thing");
			recipe.PreparationTime = new TimeSpan(34);
			recipe.Save();  
		}
		
		/// <summary>
		/// Test that a Recipe can be saved and loaded by saving a new Recipe and then load it again to check 
		/// if the loaded data match.
		/// </summary>
		/// <remarks>
		/// Fixet null ref error in loading of Ingredients on Recipe.
		/// </remarks>
		[Test]
		
		public void SaveAndLoadRecipeTest()
		{
			Recipe recipe = new Recipe("Sloppy Joe");
			recipe.Ingredients.Add(new Ingredient("ingTitle"), 23);
			recipe.Ingredients.Add(new Ingredient("ingName"), 34);
			recipe.Categories.Add("Breakfast");
			recipe.Difficulty = Difficulty.Easy;
			recipe.Directions.Add("Do the first ting");
			recipe.Directions.Add("Do the next thing");
			recipe.PreparationTime = new TimeSpan(34);
			recipe.Save();
			
			Recipe loadedrecipe =  Recipe.GetByTitle("Sloppy Joe");  
			
			Assert.AreEqual("Sloppy Joe",loadedrecipe.Title);
			Assert.AreEqual(2,loadedrecipe.Ingredients.Count);
			Assert.AreEqual(2,loadedrecipe.Directions.Count);
			
			  
		}
		/// <summary>
		/// Test the cookRecipe method by calling the method on a recipe and check that the Inventory and Ingredients match the expected.
		/// </summary>
		/// <remarks>
		/// Fixet calculating new ingredient when cooking a new FoodItem
		/// </remarks>
		[Test]
		public void CookRecipeTest()
		{
			Ingredient ing1 = Ingredient.GetByLongDescription("Butter, salted");
			Ingredient ing2 = Ingredient.GetByLongDescription("Cheese, brick");
			 
			ing1.Save();
			ing2.Save(); 
			
			FoodItem food1 = new FoodItem(80,ing1,new DateTime((int)2010,(int)1,(int)1),false);
			FoodItem food2 = new FoodItem(80,ing2,new DateTime((int)2010,(int)1,(int)1),false);
			food1.Save();
			food2.Save(); 
			
			Recipe recipe = new Recipe("Sloppy Joe");
			recipe.Ingredients.Add(ing1, 34);
			recipe.Ingredients.Add(ing2, 22);
			recipe.Categories.Add("Breakfast");
			recipe.Difficulty = Difficulty.Easy;
			recipe.Directions.Add("Do the first ting");
			recipe.Directions.Add("Do the next thing");
			recipe.PreparationTime = new TimeSpan(34);
			recipe.Save();
			
			Recipe loadedrecipe =  Recipe.GetByTitle("Sloppy Joe");  
			
			Assert.AreNotEqual(null,loadedrecipe,"must not be null");
			
			List<FoodItem> count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual(2, count.Count ,"there shoud only be 2 fooditems in the database");
			
			loadedrecipe.Cook(2,50).Save() ;
			
			Assert.AreNotEqual(null, Ingredient.GetByLongDescription("Sloppy Joe"),"there is no ingredient Sloppy Joe");
			
			Assert.AreEqual( Math.Round((ing1.Carbohydrates * 34 + ing2.Carbohydrates*22)/200 ,4),Math.Round(Ingredient.GetByLongDescription("Sloppy Joe").Carbohydrates,4) ,"add the Carbohydrates wrong"); 
			Assert.AreEqual( Math.Round((ing1.Protein * 34 + ing2.Protein*22)/200 ,4),Math.Round(Ingredient.GetByLongDescription("Sloppy Joe").Protein,4) ,"add the Protein wrong"); 
			Assert.AreEqual( Math.Round((ing1.Fat * 34 + ing2.Fat*22)/200 ,4),Math.Round(Ingredient.GetByLongDescription("Sloppy Joe").Fat,4) ,"add the Fat wrong"); 
					
			count = new List<FoodItem>(FoodItem.ListFoodItems());
			Assert.AreEqual(3, count.Count ,"there shoud be 3 fooditems in the database as we have cooked a new one");
			  
			
			List<FoodItem> fooditem = new List<FoodItem>(FoodItem.ListFoodItems());
				foreach (var f in fooditem){
					if (f.Id == food1.Id){
						Assert.AreEqual(12, f.Quantity  ,"foodremaining dont match");
					}else if (f.Id == food2.Id){
						Assert.AreEqual(36, f.Quantity  ,"foodremaining dont match");
					}else{
						Assert.AreEqual(56, f.Quantity  ,"new cooked fooditem dont match");
					}
				}
		}
	}
}
