﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Foodolini.BusinessLogic;

namespace Foodolini.Tools
{
    public abstract class RecipeImporter
    {
        public event EventHandler<RecipeImporterEventArgs> RaiseImportEvent;
        public abstract Recipe ReadData(System.IO.FileStream fs);

        protected void OnRaiseImportEvent(RecipeImporterEventArgs e)
        {
            EventHandler<RecipeImporterEventArgs> handler = RaiseImportEvent;
            if (handler != null)
                handler(this, e);
        }


        /// <summary>
        /// Converts a string-based fraction into a double.
        /// </summary>
        /// <param name="s"></param>
        /// <returns></returns>
        protected double ParseQuantity(string[] s)
        {
            double result;
            char[] frac = { '/' };
            if (s.Length > 1) {
                double a = double.Parse(s[0]);
                string[] f = s[1].Split(frac);
                result = a + (double.Parse(f[0]) / double.Parse(f[1]));
            } else {
                string[] f = s[0].Split(frac);
                if (f.Length > 1)
                    result = double.Parse(f[0]) / double.Parse(f[1]);
                else
                    result = double.Parse(f[0]);
            }
            return result;
        }
    }

    public class RecipeImporterEventArgs : EventArgs
    {
        public string Message { get; set; }
        public EventCode EvCode { get; set; }

        public RecipeImporterEventArgs(string message) : this(message, EventCode.None) { }

        public RecipeImporterEventArgs(string message, EventCode evCode)
        {
            this.Message = message;
            this.EvCode = evCode;
        }

        public enum EventCode
        {
            None,
            ServingsParseError,
            QuantityParseError,
            DataSourceError
        }
    }
}
