﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Xml;
using System.IO;
using System.Xml.Linq;
using Foodolini.BusinessLogic;

namespace Foodolini.Tools
{
    public class RecipeMLImporter : RecipeImporter
    {

        public override Recipe ReadData(System.IO.FileStream fs)
        {
            Recipe result;
            string fileName = Path.GetFileName(fs.Name);
            XmlDocument sourceXml = new XmlDocument();

            try {
                sourceXml.Load(fs);
            } catch (XmlException ex) {
                OnRaiseImportEvent(new RecipeImporterEventArgs(fileName + ": syntax error, line " + ex.LineNumber,
                    RecipeImporterEventArgs.EventCode.DataSourceError));
                return null;
            } finally {
                fs.Close();
            }

            result = new Recipe("");
            // Set Recipe Title
            result.Title = sourceXml.SelectSingleNode("recipeml/recipe/head/title").InnerText;

            // Set Recipe Servings
            XmlNode yieldNode = sourceXml.SelectSingleNode("recipeml/recipe/head/yield");
            if (yieldNode != null) {
                string[] servings = yieldNode
                    .InnerText.ToUpper().Replace(" ", "")
                    .Split(new string[] { "-", "TO" }, StringSplitOptions.RemoveEmptyEntries);

                try {
                    if (servings.Length > 1) {
                        result.Servings = ParseQuantity(new string[] { servings[servings.Length - 1] });
                    } else
                        result.Servings = ParseQuantity(servings);
                } catch {
                    OnRaiseImportEvent(
                                    new RecipeImporterEventArgs(fileName + ": servings parse error, " + servings[0],
                                        RecipeImporterEventArgs.EventCode.ServingsParseError));
                }
            }

            // Set Recipe Categories
            List<string> categories = new List<string>();
            foreach (XmlNode cat in sourceXml.SelectNodes("recipeml/recipe/head/categories/cat"))
                categories.Add(cat.InnerText);
            //result.Categories = categories;

            // Set Recipe Ingredients
            //Dictionary<Ingredient, Amount> ingredients = new Dictionary<Ingredient, Amount>();
            foreach (XmlNode ing in sourceXml.SelectNodes("recipeml/recipe/ingredients/ing")) {
                //Amount amount = new Amount();

                XmlNode amtNode = ing.SelectSingleNode("amt");
                XmlNode qtyNode = amtNode.SelectSingleNode("qty");
                XmlNode unitNode = amtNode.SelectSingleNode("unit");

                if (qtyNode != null) {
                    string[] parts = qtyNode.InnerText.Replace(".", "").Replace("\"", "").Split(new char[] { ' ' }, StringSplitOptions.RemoveEmptyEntries);

                    if (parts != null) {
                        if (parts.Length > 0) {
                            try {
                                //amount.Quantity = ParseQuantity(parts);
                            } catch (FormatException ex) {
                                OnRaiseImportEvent(
                                    new RecipeImporterEventArgs(fileName + ": quantity parse error, " + parts[0],
                                        RecipeImporterEventArgs.EventCode.QuantityParseError));
                                return null;
                            }
                        }
                    }
                }

                if (unitNode != null) {
                    //amount.Unit = unitNode.InnerText;
                }
                //ingredients.Add(new Ingredient(ing.SelectSingleNode("item").InnerText), amount);
            }
            //result.Ingredients = ingredients;

            // Set Recipe Directions
            List<string> directions = new List<string>();
            foreach (XmlNode dir in sourceXml.SelectNodes("recipeml/recipe/directions/step")) {
                foreach (string step in dir.InnerText.Split(new string[] { "\n  \n" }, StringSplitOptions.RemoveEmptyEntries)) {
                    if (step.Replace(" ", "").Length > 0)
                        directions.Add(step);
                }
            }
            //result.Directions = directions;

            return result;
        }


    }
}
